package com.pruebasaplicacion.sergimeli.bletests;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.Spinner;
import android.app.AlertDialog;

import com.pruebasaplicacion.sergimeli.bletests.MainActivity;
import com.pruebasaplicacion.sergimeli.bletests.modelo.DatosDispositivo;
import com.pruebasaplicacion.sergimeli.bletests.modelo.Usuario;
import com.pruebasaplicacion.sergimeli.bletests.sqlite.DeviceContract;
import com.pruebasaplicacion.sergimeli.bletests.sqlite.OperacionesBaseDatos;

import java.util.Calendar;

public class NewDeviceActivity extends Activity {

    private OperacionesBaseDatos datos;

    /** Interfaz de nuevo dispositivo detectado **/
    private EditText DipositiveNameEdit;
    private EditText UsersNameEdit;
    private Spinner ExistentUsersSpinner;
    private EditText UsersWeightEdit;
    private EditText UsersHeightEdit;
    private Button ConfirmButton;
    private Button CancelNewDispositiveButton;

    private String mDevice, mNombre;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_new_device);
        //Conseguir los datos que llevaba el intent
        Intent intent = getIntent();
        mDevice = intent.getStringExtra(MainActivity.SENT_MDEVICE);

        /** Asignar los valores de las views **/
        DipositiveNameEdit = (EditText)findViewById(R.id.NombreDispositivoEdit);
        UsersNameEdit = (EditText)findViewById(R.id.NombreUsuarioEdit);
        ExistentUsersSpinner = (Spinner)findViewById(R.id.DesplegableUsuariosExistentes);
        UsersWeightEdit = (EditText)findViewById(R.id.PesoUsuarioEdit);
        UsersHeightEdit = (EditText)findViewById(R.id.AlturaUsuarioEdit);
        ConfirmButton = (Button)findViewById(R.id.BotonConfirmarDatos);
        CancelNewDispositiveButton = (Button)findViewById(R.id.BotonCancelarNuevoDispositivo);

        /** Obtener una instancia de la base de datos **/
        datos = OperacionesBaseDatos.obtenerInstancia(getApplicationContext());

        Usuarios_ya_registrados();

        //Activiar un listener de onItemSelected al spinner de usuarios
        ExistentUsersSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                String[] aux = getAllExistentUsers();
                String[] aux2 = getAllIdfromExistentUsers();
                if (aux[position].equals(getString(R.string.newUserString))) {
                    UsersNameEdit.setHint(aux[position]);
                    UsersWeightEdit.setHint(getString(R.string.defaultEditValuesWeightHint));
                    UsersHeightEdit.setHint(getString(R.string.defaultEditValuesHeightHint));
                } else {
                    UsersNameEdit.setHint(aux[position]);
                    UsersWeightEdit.setHint(getSpecifiedUserWeight(aux2[position]));
                    UsersHeightEdit.setHint(getSpecifiedUserHeight(aux2[position]));
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
        /**
         * Añadir un listener OnClick al botón de confirmar
         */
        ConfirmButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String newGivenName = DipositiveNameEdit.getText().toString();
                //Comprobar que el nombre del dispositivo no es el default
                if (newGivenName.equals(getString(R.string.DeviceNameDefault))) {
                    //Si ese es el caso, vuelve todos a los valores default y muestra un mensae de alerta
                    DipositiveNameEdit.setText("");
                    UsersNameEdit.setText("");
                    UsersWeightEdit.setText("");
                    UsersHeightEdit.setText("");
                    final AlertDialog.Builder builder1 = new AlertDialog.Builder(NewDeviceActivity.this);
                    builder1.setTitle(getString(R.string.errorDialogDeviceNameTitle));
                    builder1.setMessage(getString(R.string.errorDialogDeviceNameDescription));
                    builder1.setPositiveButton(android.R.string.ok, null);
                    builder1.setOnDismissListener(new DialogInterface.OnDismissListener() {
                        @Override
                        public void onDismiss(DialogInterface dialog) {

                        }
                    });
                    builder1.show();
                } else {
                    //Comprobar que el nombre del dispositivo no esta vacio
                    if (newGivenName.equals("")){
                        //Si es así, vuelve todos a los valores default y muestra mensaje de alerta
                        DipositiveNameEdit.setText("");
                        UsersNameEdit.setText("");
                        UsersWeightEdit.setText("");
                        UsersHeightEdit.setText("");
                        final AlertDialog.Builder builder1 = new AlertDialog.Builder(NewDeviceActivity.this);
                        builder1.setTitle(getString(R.string.errorDialogDeviceNameTitle));
                        builder1.setMessage(getString(R.string.errorDialogBlankDeviceNameDescription));
                        builder1.setPositiveButton(android.R.string.ok, null);
                        builder1.setOnDismissListener(new DialogInterface.OnDismissListener() {
                            @Override
                            public void onDismiss(DialogInterface dialog) {

                            }
                        });
                        builder1.show();
                    }else {
                        String mUsersHeight = UsersHeightEdit.getText().toString();
                        String mUsersWeight = UsersWeightEdit.getText().toString();
                        String mUserName = UsersNameEdit.getText().toString();
                        //Comprobar que el nombre de usuario proporcionado no esta vacio
                        if (mUserName.equals("")){
                            //Si es así, envía una alerta de error y vuelve el nombre de usuario a su valor default
                            UsersNameEdit.setText("");
                            final AlertDialog.Builder builder1 = new AlertDialog.Builder(NewDeviceActivity.this);
                            builder1.setTitle(getString(R.string.errorDialogDeviceNameTitle));
                            builder1.setMessage(getString(R.string.errorDialogBlankUserName));
                            builder1.setPositiveButton(android.R.string.ok, null);
                            builder1.setOnDismissListener(new DialogInterface.OnDismissListener() {
                                @Override
                                public void onDismiss(DialogInterface dialog) {

                                }
                            });
                            builder1.show();
                        }else {
                            //Aqui se comprueba si el nombre de usuario ya existe en la db
                            mNombre = mUserName;
                            String[] aux = getAllExistentUsers();
                            String[] aux2 = getAllIdfromExistentUsers();
                            int position = 0;
                            boolean userexist = false;
                            for (int i = 0; i < aux.length; i++) {
                                if (aux[i].equals(mUserName)) {
                                    userexist = true;
                                    position = i;
                                    break;
                                }
                            }
                            /*
                             * Si el usuario existe, registrae el nuevo dispositivo con los datos
                             * del usuario existente.
                             */
                            if (userexist)
                                datos.insertarUsuario(new Usuario(aux2[position], mDevice,
                                        mUserName, mUsersHeight, mUsersWeight,
                                        getSpecifiedUserBadSteps(aux2[position]),
                                        getSpecifiedUserlastLog(aux2[position]),
                                        getSpecifiedUserChallenge(aux2[position]),
                                        getSpecifiedUserCompleted(aux2[position])));
                            /**
                             * Si el usuario no existe, crea un usuario nuevo con los datos del
                             * dispositivo nuevo.
                             */
                            else {
                                if(mUsersHeight.equals("")) mUsersHeight = "0";
                                if(mUsersWeight.equals("")) mUsersWeight = "0";
                                datos.insertarUsuario(new Usuario(null, mDevice, mUserName, mUsersHeight,
                                        mUsersWeight, "0", "0:0", 4, 0));
                            }

                            datos.actualizarNombreDispositivo(new DatosDispositivo(mDevice, null, null, newGivenName));
                            //Mostrar mensaje de aviso
                            final AlertDialog.Builder builder1 = new AlertDialog.Builder(NewDeviceActivity.this);
                            builder1.setTitle(getString(R.string.newDeviceWarningTitle));
                            builder1.setMessage(getString(R.string.newDeviceWarningBody));
                            builder1.setPositiveButton(android.R.string.ok, null);
                            builder1.setOnDismissListener(new DialogInterface.OnDismissListener() {
                                @Override
                                public void onDismiss(DialogInterface dialog) {
                                    //Vuelve a la actividad principal
                                    Intent intent1 = new Intent(NewDeviceActivity.this, MainActivity.class);
                                    startActivity(intent1);
                                }
                            });
                            builder1.show();


                        }
                    }
                }
            }
        });
        /**
         * Si se cancela la creación de un nuevo usuario se procederá a volver a la actividad
         * principal
         */
        CancelNewDispositiveButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent1 = new Intent(NewDeviceActivity.this, MainActivity.class);
                startActivity(intent1);
            }
        });
    }

    /** Asignar al spinner de usuarios existentes, los usuarios existentes **/
    private void Usuarios_ya_registrados(){

        String[] users = getAllExistentUsers();
        // Create an ArrayAdapter using the string array and a default spinner layout
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(this,
                android.R.layout.simple_spinner_item,users);
        // Specify the layout to use when the list of choices appears
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        // Apply the adapter to the spinner
        ExistentUsersSpinner.setAdapter(adapter);
    }

    /*  [Cosas de Usuario]
    *  El siguiente método consigue todos los usuarios registrados en la DB (con un límite
    *  de 5) y los comunica mediante un String Array
    */

    public String[] getAllExistentUsers(){
        Cursor cursor = datos.obtenerUsuarios();
        String aux, aux1 = getString(R.string.newUserString);
        String[] Users = {aux1,aux1,aux1,aux1,aux1};
        int count = Users.length -1;
        if(cursor.moveToFirst()){
            do{
                aux = cursor.getString(cursor.getColumnIndex(DeviceContract.Usuarios.NOMBRE));
                Users[count] = aux;
                count--;
            }while (cursor.moveToNext());
        }
        cursor.close();
        return Users;
    }

    /**
     * El método que hay a continuación consigue todos los id de usuarios registrados en la DB
     * (Con un límite de 5) y los comunica con un StringArray.
     */
    public String[] getAllIdfromExistentUsers(){
        Cursor cursor = datos.obtenerUsuarios();
        String aux, aux1 = getString(R.string.newUserIdString);
        String[] id = {aux1,aux1,aux1,aux1,aux1};
        int count = id.length -1;
        if(cursor.moveToFirst()){
            do{
                aux = cursor.getString(cursor.getColumnIndex(DeviceContract.Usuarios.ID));
                id[count] = aux;
                count--;
            }while (cursor.moveToNext());
        }
        cursor.close();
        return id;
    }

    public String getSpecifiedUserHeight(String id){
        Cursor cursor = datos.obtenerUsuarioPorId(id);
        String res="";

        if(cursor.moveToFirst()){
            do{
                res = cursor.getString(cursor.getColumnIndex(DeviceContract.Usuarios.ALTURA));
            }while (cursor.moveToNext());
        }
        cursor.close();
        return res;
    }

    /**
     * El siguiente método consigue el peso del usuario especificado
     */

    public String getSpecifiedUserWeight(String id){
        Cursor cursor = datos.obtenerUsuarioPorId(id);
        String res = "";

        if(cursor.moveToFirst()){
            do{
                res = cursor.getString(cursor.getColumnIndex(DeviceContract.Usuarios.PESO));
            }while(cursor.moveToNext());
        }
        cursor.close();
        return res;
    }

    public String getSpecifiedUserBadSteps(String id){
        Cursor cursor = datos.obtenerUsuarioPorId(id);
        String res = "";
        if(cursor.moveToFirst()){
            do{
                res = cursor.getString(cursor.getColumnIndex(DeviceContract.Usuarios.ZANCADA));
            }while (cursor.moveToNext());
        }
        cursor.close();
        return res;
    }

    public String getSpecifiedUserlastLog(String id){
        Cursor cursor = datos.obtenerUsuarioPorId(id);
        String res = "";
        if(cursor.moveToFirst()){
            do{
                res = cursor.getString(cursor.getColumnIndex(DeviceContract.Usuarios.LASTLOG));
            }while (cursor.moveToNext());
        }
        cursor.close();
        return res;
    }

    public int getSpecifiedUserChallenge(String id){
        Cursor cursor = datos.obtenerUsuarioPorId(id);
        int res = 0;
        if(cursor.moveToFirst()){
            do{
                res = cursor.getInt(cursor.getColumnIndex(DeviceContract.Usuarios.RETODIARIO));
            }while (cursor.moveToNext());
        }
        cursor.close();
        return res;
    }

    public int getSpecifiedUserCompleted(String id){
        Cursor cursor = datos.obtenerUsuarioPorId(id);
        int res = 0;

        if(cursor.moveToFirst()){
            do{
                res = cursor.getInt(cursor.getColumnIndex(DeviceContract.Usuarios.RETOCOMPLTADO));
            }while (cursor.moveToNext());
        }
        cursor.close();
        return res;
    }

    public String getUserName(){
        return this.mNombre;
    }

}
