package com.pruebasaplicacion.sergimeli.bletests.sqlite;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Build;
import android.provider.BaseColumns;

import com.pruebasaplicacion.sergimeli.bletests.R;
import com.pruebasaplicacion.sergimeli.bletests.modelo.RendimientoUsuario;

/**
 * Created by Elerionfly on 22/07/2016.
 */
public class BaseDatosDispositivo extends SQLiteOpenHelper {

    private static final String NOMBRE_BASE_DATOS = "devices.db";
    private static final int VERSION_ACTUAL = 1;
    private final Context context;

    interface Tablas{
        String DATOS_DISPOSITIVO = "datos_dispositivo";
        String DATOS_USUARIO = "usuario";
        String DATOS_RENDIMIENTO_USUARIO = "rendimiento_usuario";
    }

    interface Referencias{
        String ID_DATOS_DISPOSITIVO = String.format("REFERENCES %s(%s) ON DELETE CASCADE",
                Tablas.DATOS_DISPOSITIVO, DeviceContract.DatosDispositivos.MDEVICE);
        String ID_DATOS_USUARIO = String.format("REFERENCES %s(%s) ON DELETE CASCADE",
                Tablas.DATOS_USUARIO, DeviceContract.Usuarios.NOMBRE);
    }

    public BaseDatosDispositivo(Context context){
        super(context, NOMBRE_BASE_DATOS, null, VERSION_ACTUAL);
        this.context = context;
    }

    @Override
    public void onOpen(SQLiteDatabase db){
        super.onOpen(db);
        if (!db.isReadOnly()) {
            if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN){
                db.setForeignKeyConstraintsEnabled(true);
            }else{
                db.execSQL("PRAGMA foreign_keys = ON");
            }
        }
    }

    @Override
    public void onCreate (SQLiteDatabase db){
        /**
         * Crear tabla de dispositivo
         * PK: mDevice
         */

        db.execSQL(String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY AUTOINCREMENT," +
                        "%s TEXT UNIQUE NOT NULL, %s TEXT NOT NULL, %s TEXT NOT NULL," +
                        "%s TEXT NOT NULL)",
                Tablas.DATOS_DISPOSITIVO, BaseColumns._ID,
                DeviceContract.DatosDispositivos.MDEVICE,
                DeviceContract.DatosDispositivos.MSERVICEUUID, DeviceContract.DatosDispositivos.MDEVICENAME,
                DeviceContract.DatosDispositivos.GIVENNAME));

        db.execSQL(String.format("CREATE UNIQUE INDEX i2 ON %s(%s)",
                Tablas.DATOS_DISPOSITIVO, DeviceContract.DatosDispositivos.MDEVICE));

        /**
         * Crear tabla de Usuario
         * PK: ID de Usuario
         * FK: mDevice
         */

        db.execSQL(String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY AUTOINCREMENT," +
                        "%s TEXT UNIQUE NOT NULL, %s TEXT NOT NULL %s, %s TEXT NOT NULL, %s TEXT NOT NULL," +
                        "%s TEXT NOT NULL, %s TEXT NOT NULL, %s TEXT NOT NULL, %s INTEGER NOT NULL," +
                        "%s INTEGER NOT NULL)",
                Tablas.DATOS_USUARIO, BaseColumns._ID,
                DeviceContract.Usuarios.ID, DeviceContract.Usuarios.USUARIO_DEVICE,
                Referencias.ID_DATOS_DISPOSITIVO,
                DeviceContract.Usuarios.NOMBRE, DeviceContract.Usuarios.ALTURA,
                DeviceContract.Usuarios.PESO, DeviceContract.Usuarios.ZANCADA,
                DeviceContract.Usuarios.LASTLOG, DeviceContract.Usuarios.RETODIARIO,
                DeviceContract.Usuarios.RETOCOMPLTADO));

        /**
         * Indices tabla usuario
         */

        db.execSQL(String.format("CREATE UNIQUE INDEX i1 ON %s(%s)",
                Tablas.DATOS_USUARIO, DeviceContract.Usuarios.NOMBRE));

        /**
         * Crear tabla de Rendimiento
         * PK: ID de Rendimiento
         * FK: Nombre
         */

        db.execSQL(String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY AUTOINCREMENT," +
                        "%s TEXT UNIQUE NOT NULL, %s TEXT NOT NULL %s, %s TEXT NOT NULL," +
                        "%s INTEGER NOT NULL, %s INTEGER NOT NULL, %s INTEGER NOT NULL," +
                        "%s INTEGER NOT NULL," +
                        "%s REAL NOT NULL, %s REAL NOT NULL)",
                Tablas.DATOS_RENDIMIENTO_USUARIO, BaseColumns._ID,
                DeviceContract.RendimientoUsuarios.ID, DeviceContract.RendimientoUsuarios.NOMBRE,
                Referencias.ID_DATOS_USUARIO,
                DeviceContract.RendimientoUsuarios.FECHA,
                DeviceContract.RendimientoUsuarios.PASOS, DeviceContract.RendimientoUsuarios.PASOS_MALOS,
                DeviceContract.RendimientoUsuarios.PASOS_MALOS_2, DeviceContract.RendimientoUsuarios.PASOS_PESADOS,
                DeviceContract.RendimientoUsuarios.DISTANCIA, DeviceContract.RendimientoUsuarios.CALORIAS_QUEMADAS));

    }

    @Override
    public void onUpgrade (SQLiteDatabase db, int oldVersion, int newVersion){
        db.execSQL("DROP TABLE IF EXISTS " + Tablas.DATOS_DISPOSITIVO);
        db.execSQL("DROP TABLE IF EXISTS " + Tablas.DATOS_USUARIO);
        db.execSQL("DROP TABLE IF EXISTS " + Tablas.DATOS_RENDIMIENTO_USUARIO);

        onCreate(db);
    }

    public static int getVersionActual() {
        return VERSION_ACTUAL;
    }

    public void DeletePerformance(SQLiteDatabase db){
        db.execSQL("DELETE FROM " + Tablas.DATOS_RENDIMIENTO_USUARIO);
    }
}
